==> kwin-gestures-debug-0.5.0-1-x86_64.pkg.tar.zst: usr/src/debug/kwin-gestures/kwin-gestures-0.5.0/src/kwin/impl/kwininput.cpp <==
#include "kwininput.h"

#include "utils.h"

#include "keyboard_input.h"
#include "pointer_input.h"

// TODO move key list
#include "libgestures/yaml_convert.h"

KWinInput::KWinInput()
    : m_device(std::make_unique<InputDevice>())
{
    m_input = KWin::input();
    m_input->addInputDevice(m_device.get());
    m_pointer = m_input->pointer();
    m_keyboard = m_input->keyboard();

    connect(m_input, &KWin::InputRedirection::keyboardModifiersChanged, this, &KWinInput::slotKeyboardModifiersChanged);
    connect(m_input, &KWin::InputRedirection::keyStateChanged, this, &KWinInput::slotKeyStateChanged);
}

KWinInput::~KWinInput()
{
    if (KWin::input()) {
        KWin::input()->removeInputDevice(m_device.get());
    }
}

void KWinInput::keyboardKey(const uint32_t &key, const bool &state)
{
    m_ignoreModifierUpdates = true;
    m_keyboard->processKey(key, state ? KeyboardKeyStatePressed : KeyboardKeyStateReleased, timestamp(), m_device.get());
    m_ignoreModifierUpdates = false;
}

Qt::KeyboardModifiers KWinInput::keyboardModifiers() const
{
    return m_modifiers;
}

void KWinInput::keyboardClearModifiers()
{
    // These events will belong to a different device, which wouldn't work with normal keys, but it works with modifiers.
    // The user should be able to start the gesture again while still having the modifiers pressed, so the previous
    // ones must be kept track of.
    keyboardKey(KEY_LEFTALT, false);
    keyboardKey(KEY_LEFTCTRL, false);
    keyboardKey(KEY_LEFTMETA, false);
    keyboardKey(KEY_LEFTSHIFT, false);
    keyboardKey(KEY_RIGHTALT, false);
    keyboardKey(KEY_RIGHTCTRL, false);
    keyboardKey(KEY_RIGHTMETA, false);
    keyboardKey(KEY_RIGHTSHIFT, false);
}

void KWinInput::mouseButton(const uint32_t &button, const bool &state)
{
    m_pointer->processButton(button, state ? PointerButtonStatePressed : PointerButtonStateReleased, timestamp(), m_device.get());
    m_pointer->processFrame(m_device.get());
}

void KWinInput::mouseMoveAbsolute(const QPointF &pos)
{
    m_pointer->processMotionAbsolute(pos, timestamp(), m_device.get());
    m_pointer->processFrame(m_device.get());
}

void KWinInput::mouseMoveRelative(const QPointF &pos)
{
    m_pointer->processMotion(pos, pos, timestamp(), m_device.get());
    m_pointer->processFrame(m_device.get());
}

void KWinInput::slotKeyboardModifiersChanged(Qt::KeyboardModifiers newMods, Qt::KeyboardModifiers oldMods)
{
    if (m_ignoreModifierUpdates) {
        return;
    }

    m_modifiers = newMods;
}

void KWinInput::slotKeyStateChanged(quint32 keyCode, KeyboardKeyState state)
{
    if (m_ignoreModifierUpdates || state != KeyboardKeyStateReleased) {
        return;
    }

    if (keyCode == KEY_LEFTALT || keyCode == KEY_RIGHTALT) {
        m_modifiers &= ~Qt::KeyboardModifier::AltModifier;
    } else if (keyCode == KEY_LEFTCTRL || keyCode == KEY_RIGHTCTRL) {
        m_modifiers &= ~Qt::KeyboardModifier::ControlModifier;
    } else if (keyCode == KEY_LEFTMETA || keyCode == KEY_RIGHTMETA) {
        m_modifiers &= ~Qt::KeyboardModifier::MetaModifier;
    } else if (keyCode == KEY_LEFTSHIFT || keyCode == KEY_RIGHTSHIFT) {
        m_modifiers &= ~Qt::KeyboardModifier::ShiftModifier;
    }
}

QString InputDevice::name() const
{
    return "kwin_gestures";
}

bool InputDevice::isEnabled() const
{
    return true;
}

void InputDevice::setEnabled(bool enabled)
{
    Q_UNUSED(enabled)
}

bool InputDevice::isKeyboard() const
{
    return true;
}

bool InputDevice::isPointer() const
{
    return true;
}

bool InputDevice::isTouchpad() const
{
    return false;
}

bool InputDevice::isTouch() const
{
    return false;
}

bool InputDevice::isTabletTool() const
{
    return false;
}

bool InputDevice::isTabletPad() const
{
    return false;
}

bool InputDevice::isTabletModeSwitch() const
{
    return false;
}

bool InputDevice::isLidSwitch() const
{
    return false;
}

#ifndef KWIN_6_3_OR_GREATER
QString InputDevice::sysName() const
{
    return name();
}

KWin::LEDs InputDevice::leds() const
{
    return KWin::LEDs::fromInt(0);
}

void InputDevice::setLeds(KWin::LEDs leds)
{
    Q_UNUSED(leds)
}
#endif

==> kwin-gestures-debug-0.5.1-1-x86_64.pkg.tar.zst: usr/src/debug/kwin-gestures/kwin-gestures-0.5.1/src/kwin/impl/kwininput.cpp <==
#include "kwininput.h"

#include "utils.h"

#include "keyboard_input.h"
#include "pointer_input.h"
#include "workspace.h"

// TODO move key list
#include "libgestures/yaml_convert.h"

KWinInput::KWinInput()
    : m_device(std::make_unique<InputDevice>())
{
    m_input = KWin::input();
    m_input->addInputDevice(m_device.get());
    m_pointer = m_input->pointer();
    m_keyboard = m_input->keyboard();

    // KWin::InputRedirection::keyboardModifiersChanged sometimes doesn't get emitted and I have no idea why, this one
    // works though.
    connect(m_input, &KWin::InputRedirection::keyStateChanged, this, &KWinInput::slotKeyStateChanged);
}

KWinInput::~KWinInput()
{
    if (KWin::input()) {
        KWin::input()->removeInputDevice(m_device.get());
    }
}

void KWinInput::keyboardKey(const uint32_t &key, const bool &state)
{
    m_ignoreModifierUpdates = true;
    m_keyboard->processKey(key, state ? KeyboardKeyStatePressed : KeyboardKeyStateReleased, timestamp(), m_device.get());
    m_ignoreModifierUpdates = false;
}

Qt::KeyboardModifiers KWinInput::keyboardModifiers() const
{
    return m_modifiers;
}

void KWinInput::keyboardClearModifiers()
{
    // Prevent modifier-only global shortcuts from being triggered. Clients will still see the event and may perform
    // actions.
    const auto globalShortcutsDisabled = KWin::workspace()->globalShortcutsDisabled();
    if (!globalShortcutsDisabled) {
        KWin::workspace()->disableGlobalShortcutsForClient(true);
    }

    // These events will belong to a different device, which wouldn't work with normal keys, but it works with modifiers.
    // The user should be able to start the gesture again while still having the modifiers pressed, so the previous
    // ones must be kept track of.
    keyboardKey(KEY_LEFTALT, false);
    keyboardKey(KEY_LEFTCTRL, false);
    keyboardKey(KEY_LEFTMETA, false);
    keyboardKey(KEY_LEFTSHIFT, false);
    keyboardKey(KEY_RIGHTALT, false);
    keyboardKey(KEY_RIGHTCTRL, false);
    keyboardKey(KEY_RIGHTMETA, false);
    keyboardKey(KEY_RIGHTSHIFT, false);

    if (!globalShortcutsDisabled) {
        KWin::workspace()->disableGlobalShortcutsForClient(false);
    }
}

void KWinInput::mouseButton(const uint32_t &button, const bool &state)
{
    m_pointer->processButton(button, state ? PointerButtonStatePressed : PointerButtonStateReleased, timestamp(), m_device.get());
    m_pointer->processFrame(m_device.get());
}

void KWinInput::mouseMoveAbsolute(const QPointF &pos)
{
    m_pointer->processMotionAbsolute(pos, timestamp(), m_device.get());
    m_pointer->processFrame(m_device.get());
}

void KWinInput::mouseMoveRelative(const QPointF &pos)
{
    m_pointer->processMotion(pos, pos, timestamp(), m_device.get());
    m_pointer->processFrame(m_device.get());
}

void KWinInput::slotKeyStateChanged(quint32 keyCode, KeyboardKeyState state)
{
    if (m_ignoreModifierUpdates) {
        return;
    }

    Qt::KeyboardModifier modifier;
    switch (keyCode) {
        case KEY_LEFTALT:
        case KEY_RIGHTALT:
            modifier = Qt::KeyboardModifier::AltModifier;
            break;
        case KEY_LEFTCTRL:
        case KEY_RIGHTCTRL:
            modifier = Qt::KeyboardModifier::ControlModifier;
            break;
        case KEY_LEFTMETA:
        case KEY_RIGHTMETA:
            modifier = Qt::KeyboardModifier::MetaModifier;
            break;
        case KEY_LEFTSHIFT:
        case KEY_RIGHTSHIFT:
            modifier = Qt::KeyboardModifier::ShiftModifier;
            break;
        default:
            return;
    }

    switch (state) {
        case KeyboardKeyStatePressed:
            m_modifiers |= modifier;
            break;
        case KeyboardKeyStateReleased:
            m_modifiers &= ~modifier;
            break;
        default:
            break;
    }
}

QString InputDevice::name() const
{
    return "kwin_gestures";
}

bool InputDevice::isEnabled() const
{
    return true;
}

void InputDevice::setEnabled(bool enabled)
{
    Q_UNUSED(enabled)
}

bool InputDevice::isKeyboard() const
{
    return true;
}

bool InputDevice::isPointer() const
{
    return true;
}

bool InputDevice::isTouchpad() const
{
    return false;
}

bool InputDevice::isTouch() const
{
    return false;
}

bool InputDevice::isTabletTool() const
{
    return false;
}

bool InputDevice::isTabletPad() const
{
    return false;
}

bool InputDevice::isTabletModeSwitch() const
{
    return false;
}

bool InputDevice::isLidSwitch() const
{
    return false;
}

#ifndef KWIN_6_3_OR_GREATER
QString InputDevice::sysName() const
{
    return name();
}

KWin::LEDs InputDevice::leds() const
{
    return KWin::LEDs::fromInt(0);
}

void InputDevice::setLeds(KWin::LEDs leds)
{
    Q_UNUSED(leds)
}
#endif
